;: Title: DDB by HotKeyIt

;
; Function: Introduction
; Description:
;		Welcome to DDB, the ultimate array and dictionary function for AutoHotkey.
; Syntax: DDB is based on COM dictionary object and it is very easy to use.<br>DDB can be used in three ways.<br><br> - Directly using DDB function<br> - Trough ArrayFunctions<br> - Trough DictionaryFunctions<br><br>To use Array- and DictionaryFunctions functions you will need to define an empty function with predefined parameters.<br><br>After DDB() has been called, those empty functions will become functional arrays and dictionaries.
; Parameters:
;		<b>Function</b> - <b>Usage</b>
;		DDB - You can call DDB(dictionary,action,key,item) function directly to save, get, find, delete... values, see DDB for more information.
;		ArrayFunctions - This is a special feature of DDB, it provides very short and simple syntax.<br>To use that feature, you will need to include an empty function in your script having 1 parameter where you can set delimiter character. For example:<br>a(p=","){<br>}<br>DDB will breathe life into this function by assigning dynamically created dictionary and the code to it.
;		DictionaryFunctions - This is another special feature of DDB, it also provides similar short and simple syntax.<br>Different to ArrayFunctions, you will need to have 3 parameters in your function. For example:<br>d(key="",item="1",cmode="1"){<br>}<br>
;		Multithreading - DDB also supports multithreading<br>To use that feature you will need do get AutoHotkey.dll (1.0.48.03).<br>You can use Thread() function to start new threads.<br>In your new thread you will need to include same empty functions.<br>DDB will recognize it, check if a dictionary already exists for it and create required code as well as connect the function to the dictionary.<br>Of course you can exclude some of the functions and create new once in your new thread.
; Return Value:
;		Dynamic
; Remarks:
;		Only AutoHotkey 1.0.48.3 is supported at the moment.<br><br>To use ArrayFunctions and DictionaryFunctions you need to call DDB() once to initialize and once before exiting the process to terminate.<br><br>When using DDB in multithread environment, new functions need to have a different name than the ones in main process and other threads!<br>Note, when you terminate DDB in your thread, it will be terminated for the whole process!<br><br>You can access Array and Dictionary functions using DDB as well, therefor dic parameter must be the name of your function.
; Related:
; Example:
;		file:DDBIntroexample.ahk
;

;
; Function: DDB
; Description:
;      Dictionary Database based on Scripting.Dictionary object
; Syntax: DDB([dic,action,key,item,skip,limit,data,datakey,dataitem])
; Parameters:
;		<b>Parameter</b> - <b>Info</b>
;	   dic - Name of dictionary.<br>Name can contain only valid AutoHotkey variable characters
;		action - Action to perform
;		key - Key is used to receive values (for some actions it is used as delimiter, see below)
;		Item - Value/data to save for key (for some actions it is used as delimiter, see below)
;		Skip - When searching for keys, you can skip found items
;		Limit - Use this to find only certain amount of keys
;		Data + DataKey + DataItem - Internal parameters for ArrayFunction and DictionaryFunction
;	   <b>Action</b> - <b>Description</b>
;	   Create - Initializes COM (if not already done so) and creates a new dictionary.<br>Key can be used to set compare mode of dictionary.<br> &nbsp&nbsp- 0 = case sensitive<br> &nbsp&nbsp- 1 = not case sensitive<br> &nbsp&nbsp- 2 = database<br><br>Return value is the pointer to a dictionary. (It is used only internally so you won't need it)
;	   Count - Returns counted key+item pairs in a dictionary.
;	   Destroy - Destroy a dictionary.
;	   DestroyAll - Destroy all dictionaries.<br> &nbsp&nbsp- COM will be not terminated, use DDB() therefore
;	   GetCompMode - Returns compare mode of the dictionary
;	   Add - Adds a key and item pair to a dictionary (key is like a variable and item like value)<br> &nbsp&nbsp- If key exists, it won't be not overwritten
;	   Set - Overwrites an existing Item to new value using a key..<br> &nbsp&nbsp- If key does not exist it will be created<br>
;	   Exists - Checks if a key exist and returns 1 if so. Will return 0 if dictionary does not exist.
;	   Delete - Delete a key and its item from dictionary.
;	   Rename - Rename a key, item must contain new name of key..<br> &nbsp&nbsp- Will fail if the key given in item already exist.
;	   DeleteAll - Delete all keys and items from dictionary.
;	   Hash - Return hash of key
;	   FIND / MATCH / REGEX / GETALL - Parameter skip can be used to omit first found items<br>Parameter limit can be used to limit found items<br> &nbsp&nbsp- Returns a sting containing found items
;	   GetAll - Returns a string containing all keys from dictionary<br> &nbsp&nbsp- a delimiter may be specified in key
;	   Find - Returns a string containing keys that match given characters in beginning<br> &nbsp&nbsp- a delimiter may be specified in item
;	   Match - Returns a string containing keys that match given characters anywhere inside the key<br> &nbsp&nbsp- a delimiter may be specified in item
;	   RegEx - Returns a string containing keys using RegEx function<br> &nbsp&nbsp- a delimiter may be specified in item
;	   Sort - Sort dictionary.<br> &nbsp&nbsp- Key can contain same options as Sort command in AutoHotkey<br>!!! Sort functionality is using delimiter Chr(3) + Chr(2) because of this you cannot use Sort if your keys or values contain those characters !!!
;	   Copy - Copy a dictionary to new dictionary (key parameter needs to contain a name for new dictionary)
;	   Replace - Replace values in all items.
;	   RegExReplace - Replace values in all items using RegExReplace..
;	   Enum - This action is used to loop trough keys in dictionary<br> &nbsp&nbsp- Return key by key. ErrorLevel will contain EnumPointer!
;	   Enumerate - This action is same as Enum but is more useful in while command.<br>Returns EnumPointer, ErrorLevel contains found key.<br>While(DDB("mydb","enumerate"))<br>&nbsp&nbsp&nbspkey:=ErrorLevel
; Return Value:
;		Dynamic - see actions
; Remarks:
;	    DDB is used by ArrayFunction and DictionaryFunction internally but can be used directly as well.
; Related: [bbcode]None.
; Example:
;		file:DDBExample.ahk
;

;
; Function: ArrayFunctions
; Description:
;      ArrayFunctions provide very simple and short syntax.<br>Key is 0 based and will be managed automatically inside DDB function, so you have access to items only.<br>To create such an array you have to include an empty function in your script. For example:<br>array(1=","){<br>}<br>Instead of "array" you can use any name and as many of those as you like.<br>The function must have 1 parameter with default delimiter you want to use, if empty each character will be saved in separate item!
; Syntax: array([parameter])
; Parameters:
;		<b>Parameter</b> - <b>Info</b>
;	   parameter - is used dynamically and can be<br>&nbsp&nbsp- empty to get complete array<br>&nbsp&nbsp- number to get an item from array (first item is 0)<br>&nbsp&nbsp- action can be one of the action below + restriction + delimiter<br><br>If you leave this parameter empty when defining a function, array will become a character array, which means that each character is saved in separate item, otherwise you should set default parameter of array function to a delimiter you want to use in your array, e.g. "," so you can add items array("+item1,item2,item3")
;	   <b>Action</b> - <b>Prefix operators</b>
;		Delete - - or D will delete an item
;		Add - + or A will add an item to array
;		Count - ? or C will return counted items or check if an item exist<br>To check for existence of an item call array("?10")
;		Blank - & or B will delete whole array
;		Insert - # or I will insert items
;		Get - $ or G to get items
;		Set - = or S will set items
; Return Value:
;		Dynamic - see actions
; Remarks:
;	    Name of parameters in your empty function can be any name or even a number. (e.g. array(1=",")...<br><br>You can use ":" to limit output or deletion of items, for example get item from 0 to 5 "$0:5" or just "$:5".<br><br>Last item of action can be used as a delimiter of returned string, for example "$`n" will split output by linefeed. Please note, delimiter cannot be a number!<br><br>"-" will not delete anything, if you like to delete whole array, use "&" or "b".
; Related: [bbcode]None.
; Example:
;		file:DDBAExample.ahk
;

;
; Function: DictionaryFunctions
; Description:
;		DictionaryFunctions provide also a very simple syntax and must be declared as empty function.<br> &nbsp&nbsp- Different to ArrayFunction you need to specify 3 parameters<br> &nbsp&nbsp- second parameter can be changed to a valued parameter, see more below and example<br> &nbsp&nbsp- third parameter can be used to define compare mode of dictionary (0 = case sense, 1 = no case sense, 2 = database)<br>For example:<br>dictionary(var="",value="1",cmode="1"){<br>}
; Syntax: dictionary([key="",item="",cmode=""])
; Parameters:
;		<b>Parameter</b> - <b>Info</b>
;		key - Used to save or return value from dictionary. Like a variable in AutoHotkey.
;		item - Used to save value for key.<br> &nbsp&nbsp- This parameter has special usage when key parameter is empty.<br> &nbsp&nbsp- If this parameter is not empty in function definition, it will be automatically changed to ""<br> &nbsp&nbsp&nbsp Then you can set items blank/empty using dictionary(var,"")
;		CMode - Can be used to define delimiter for returned string when using Find/Match/GetAll...
;		<b>Action</b> - <b>Prefix operators</b>
;		Empty - / or 0 or E will empty an item<br> - When second parameter is not empty in function definition dictionary(key,"") will also empty an item.
;		Delete - - or 1 or D will delete an item<br>Third parameter must be the key you want to delete
;		Count - ? or 2 or C will return counted items or check if a key exists.<br> - To check if a key exist include it in third parameter, e.g. dictionary("",2,"key")
;		Rename - ~ or 3 or R rename a key.<br>Third parameter will be the new name
;		Loop enum -  or 4 or L will enumerate dictionary<br> &nbsp&nbsp- Returns key by key and penum is saved in ErrorLevel
;		Find - = or 5 or F will find a key<br> &nbsp&nbsp- For example if you search for 1 dictionary("",5 1), it will match if key is 10 but not if 01<br> &nbsp&nbsp- When only 1 character is given, complete dictionary will be returned<br> &nbsp&nbsp- last (3-rd) parameter can be used as delimiter for returned string when using Find, Match, Regex and GetAll
;		Match - % or 6 or M will match a key<br> &nbsp&nbsp- Same as above but 1 also matches if key is 01
;		Regex - * or 7 or X will use regex to match keys
;		GetAll - ! or 8 or G will return all keys in one string
;		While enum - # or 9 or W	will enumerate trough dictionary<br> &nbsp&nbsp- Returns penum and key is saved in ErrorLevel
;		Blank - & or B will delete complete dictionary
;		Replace - > or I will perform a replace on all items
;		RegExReplace - < or P will perform RegExReplace on all items
; Return Value:
;		Dynamic - see actions
; Remarks:
;	    Name of parameters in your empty function can be any name or even numbers ( e.g. dic(1="",2="1",3="1") ).<br><br>To use avalable actions you have to leave first parameter empty and add a prefix in front of second parameter, third parameter can be used as output delimiter.<br>For example:<br>&nbsp&nbsparray("","?key") <br>&nbsp&nbsparray("",6 var,"`n")
; Related: [bbcode]None.
; Example:
;		file:DDBDexample.ahk
;

;
; Function: Thread
; Description:
;      Multithreading is based on AutoHotkey.dll (only <a href=http://www.autohotkey.net/~tinku99/ahkdll/release/AutoHotkeyC48N8.zip>AutoHotkey.dll 1.0.48.3</a> is supported at the moment).<br>Thread function will help you to use multithreading.
; Syntax: Thread(file_or_text,[parameters,dllpath])
; Parameters:
;		<b>Parameter</b> - <b>Info</b>
;	   file_or_text - Thread can execute files as well as text. So this parameter can be a file path or text.<br><br>Otherwise this parameter can be the thread ID, see parameter for available actions.
;		parameters - You can pass on parameters to your thread as usual.<br><br>When file_or_text is a thread ID this parameter can be following value<br> - Priority or P or 1, changes priority of thread. dllpath must contain new priority<br>&nbsp&nbsp&nbsp-15&nbsp&nbsp&nbspIdle<br>&nbsp&nbsp&nbsp&nbsp&nbsp-2&nbsp&nbsp&nbspLowest<br>&nbsp&nbsp&nbsp&nbsp&nbsp-1&nbsp&nbsp&nbspBelow normal<br>&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp0&nbsp&nbsp&nbspNormal<br>&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp1&nbsp&nbsp&nbspAbove normal<br>&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp2&nbsp&nbsp&nbspHighest<br>&nbsp&nbsp&nbsp&nbsp15&nbsp&nbsp&nbspCritical<br> - Kill or K or 0, terminates a thread<br> - Suspend or S or 2, pause a thread<br> - Resume or R or 3, resume paused thread
;		dllpath - Path of AutoHotkey.dll.<br>You can leave it empty if AutoHotkey.dll exist in scripts directory, AutoHotkey.exe path or Lib folder.

; Return Value:
;		Thread returns a thread id that can be used to terminate the thread.<br>ErrorLevel will contain the pointer to the library.
; Remarks:
;	    After calling Thread, ErrorLevel contains pointer to the loaded library, unfortunately it is not possible to FreeLibrary at the moment.<br>Read about <a href=http://msdn.microsoft.com/en-us/library/ms684261.aspx>Multithreading Consideration on MSDN</a>
; Related: [bbcode]None.
; Example:
;		file:DDBMultithread.ahk
;

;
; Function: Credits
; Description:
;      Many thanks Chris for AutoHotkey and everybody who helped me building that function.
; Syntax:  Following functions were especially useful. Many thanks.
; Parameters:
;		Sean - for <a href=http://www.autohotkey.com/forum/viewtopic.php?t=17838>Scripting.Dictionary</a>.
;	   Lexikos - for <a href=http://www.autohotkey.com/forum/viewtopic.php?t=26300>LowLevel</a> fuctionality and functions, as well es example functions and help with code.
;		Laszlo - for modifications of <a href=http://www.autohotkey.com/forum/viewtopic.php?t=17838&start=11>Scripting.Dictionary</a>.
;		Tinku99 - for <a href=http://www.autohotkey.com/forum/viewtopic.php?t=43049&start=0>AutoHotkey.dll</a>, especially for <a href=http://www.autohotkey.net/~tinku99/ahkdll/release/AutoHotkeyC48N8.zip>1.0.48.3 version</a>.
;		Fincs - for <a href=http://www.autohotkey.com/forum/viewtopic.php?t=22548&highlight=gendocs>GenDocs</a>
; Return Value:
;		DDB and Thread ;)
; Remarks:
;	   DDB was not tested enough yet, so I would be really pleased to get some feedback and errors/problems report.
; Related: [bbcode]None.
; Example:
;		file:
;

DDB(dic="",action="",Key="",Item="",skip=0,limit=9223372036854775807,ByRef data="",ByRef datakey="",ByRef dataItem=""){
	static
	local CLSID,IID,CLSID_,IID_,nLen,nLenItm,bExist,label,penum,result,nCompMode,wStr,wKey,wItem,pFunc,n,MaxParams
			,split,split1,split2,Array_var,eline,pgit,Name,pName,pvName,this_postfix,vName,func_deref
	static CLSIDString:="{EE09B103-97E0-11CF-978F-00A02463E06F}", IIDString:="{42C642C1-97E1-11CF-978F-00A02463E06F}"
			,CLSID_StdGlobalInterfaceTable:="{00000323-0000-0000-C000-000000000046}"
			,IID_IGlobalInterfaceTable:="{00000146-0000-0000-C000-000000000046}"
			,_sizeof_FuncParam:=16, _JumpToLine:=4, _Param:=8, SYM_INVALID:=56, SYM_VAR:=3, SYM_FUNC:=55
			, ACT_RETURN := 102, count := 0,SYM_DYNAMIC:=5
	If (dic<>"" and action=""){
		If (dataKey<>""){
			key:=dataKey
			If ((valued_%dic% . dataItem="") or dataItem="")
				Goto, DDBGet
			Item:=dataItem
			Goto, DDBSet
		} else If ((valued_%dic% . dataItem="") or dataItem=""){
			   If (data=""){
					Goto, DDBFindAll
			   } else If data is digit
			   {
					Key:=data
					Goto,DDBGet
				} else if IsLabel(label:= "DDBA" . SubStr(data,1,1)){
					StringTrimLeft,data,data,1
					Goto % label
				} else {
					StringReplace,data,data,% delimiter_%dic%,|,All
					Item:=SubStr(data,0)
					If Item is not digit
						StringTrimRight,data,data,1
					else
						Item=
					 Key:="^(" . data . ")$"
					 label=DDBRegex
					 Goto, DDBRegex
			  }
		} else {
			Key:=SubStr(dataItem,2)
			Item:=data
			If IsLabel(label:= "DDBD" . SubStr(dataItem,1,1))
				Goto % label
		}
	} else if IsLabel(label:="DDB" . action)
		Goto % label
	else
		Return
	Return
	DDBD|:
	DDBDS:
		Item:=data, label:="DDBSort"
		Goto, %label%
	DDBD<:
	DDBDP:
		Item:=data, label:="DDBRegExReplace"
		Goto, %label%
	DDBD>:
	DDBDI:
		key:=dataItem,Item:=data, label:="DDBReplace"
		Goto, %label%
	DDBD~:
	DDBD3:
	DDBDR:
		Item:=data
		Goto, DDBRename
	DDBDG:
	DDBD!:
	DDBD8:
		key:=data
		label=DDBGetAll
		Goto, %label%
	DDBDX:
	DDBD*:
	DDBD7:
		label := (Item:=data) ? "DDBRegex" : "DDBFindAll"
		Goto,%label%
	DDBDM:
	DDBD%:
	DDBD6:
		label := (Item:=data) ? "DDBMatch" : "DDBFindAll"
		Goto,%label%
	DDBDF:
	DDBD=:
	DDBD5:
		label := (Item:=data) ? "DDBFind" : "DDBFindAll"
		Goto,%label%
	DDBDW:
	DDBD#:
	DDBD9:
		Goto, DDBEnumerate
	DDBDL:
	DDBD$:
	DDBD4:
		Goto, DDBEnum
	DDBDB:
	DDBD&:
		Goto, DDBDeleteAll
	DDBDE:
	DDBD/:
		Goto, DDBSet
	DDBDC:
	DDBD2:
	DDBD?:
	If (""!=key:=data)
		Goto, DDBExists
	else
		Goto, DDBCount
	DDBAC:
	DDBA?:
	If (""!=(key:=data))
		Goto, DDBExists
	else
		Goto,DDBCount
	DDBAG:
	DDBA$:
		If (data!=""){
			dwCookie:=SubStr(data,0)
			If dwCookie is not digit
				StringTrimRight,data,data,1
			else dwCookie:=
			If (data!=""){
				If data is digit
					n:=data,nLenItm:=limit
				else {
					RegExMatch(data,"^(\d.*?)?:?(\d.*?)?$",split)
					n:=split1,nlenItm:=split2
					If n=
						n:=0
					If nLenItm=
						nLenItm:=limit
				}
			} else n:=0,nLenItm:=limit
		}
		While % (A_Index-1<=nLenItm) {
			If (A_Index-1<n)
				continue
			Key:=A_Index-1
			Gosub, DDBExists
			NumPut(0,var1,8)
			If !bExist
				break
			Item=
			Gosub,DDBGet
			NumPut(0,var1,8),NumPut(0,var2,8)
			result .= Item . dwCookie
		}
	If (dwCookie!="")
		StringTrimRight,result,result,1
	Return result
	DDBAS:
	DDBA=:
	DDBAI:
	DDBA#:
			If (data!=""){
				If InStr(data,":"){
					RegExMatch(data,"^(\d.*?)?:",split)
					n:=split1
					If n is not digit
						n:=0
					If (n="" or StrLen(data)=StrLen(n))
						n:=0
					Else
						StringTrimLeft,data,data,% InStr(data,":")
				} else n:=0
			} else n:=0
		Gosub,DDBCount
		If (n>result)
			key:=n:=result
		else
			Key:=n
		Gosub, DDBExists
		NumPut(0,var1,8)
		If (InStr("DDBA#DDBAI",A_ThisLabel) and bExist){
			bExist=
			Loop,Parse,data,% delimiter_%dic%
				 If (SubStr(A_LoopField,0)!="``")
					  pFunc++
			Key:=result-1,Item:=result+pFunc-1
			Loop % (result-n){
				 Gosub, DDBRename
				 NumPut(0,var1,8),NumPut(0,var2,8)
				 Key--
				 Item--
			}
		} 
		result=
		Loop,parse,data,% delimiter_%dic%
		{
			Item:=A_LoopField
			key:=n
			Gosub, DDBSet
			NumPut(0,var1,8),NumPut(0,var2,8)
			n++
		}
		If (InStr("DDBA#DDBAI",A_ThisLabel)){
			key:="N",result:="",Item:="",label:="DDBSort"
			Gosub, %label%
		}
	Return
	DDBAA:
	DDBA+:
		Gosub, DDBCount
		key:=result
		Loop,Parse,data,% delimiter_%dic%
		{
			Item:=A_LoopField
			Gosub, DDBAdd
			NumPut(0,var1,8),NumPut(0,var2,8)
			Key++
		}
	Return
	DDBAB:
	DDBA&:
		GoTo,DDBDeleteAll
	DDBAD:
	DDBA-:
		If !InStr(data,","){
			If (data!=""){
				If data is digit
					n:=data,nLenItm:=limit
				else {
					RegExMatch(data,"^(\d.*?)?:?(\d.*?)?$",split)
					n:=split1,nlenItm:=split2
					If n=
						n:=0
					If nLenItm=
						nLenItm:=limit
				}
			} else n:=0,nLenItm:=limit
			Gosub, DDBCount
			if (nLenItm>result)
				nLenItm:=result
			While % ((A_Index-1+n)<nLenItm){
				key:=(n)
				Gosub, DDBDelete
				NumPut(0,var1,8)
				n++
			}
		} else {
			Loop,Parse,data,% delimiter_%dic%
			{
				Key:=A_LoopField
				Gosub, DDBDelete
				NumPut(0,var1,8)
			}
		}
		DllCall(NumGet(NumGet(pDic%dic%+0)+80), "UInt",pDic%dic%, "UIntP",penum) ; create key-list in penum
		While % (penum 
				&& !(InStr("Clear0",pDic%dic%) 
				|| DllCall(NumGet(NumGet(penum+0)+12), "UInt",penum, "UInt",1, "UInt",&var1, "UInt",0))){
			nLen := DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "UInt",0, "Int",0, "UInt",0, "UInt",0)
			VarSetCapacity(key, nLen)
			DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "Str",key, "Int",nLen, "UInt",0, "UInt",0)
			DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
			If (Key=A_Index-1)
				continue
			Item:=A_Index-1
			Gosub,DDBAllocKey
			Gosub,DDBAllocItm
			DllCall(NumGet(NumGet(pDic%dic%+0)+56), "UInt",pDic%dic%, "UInt",&var1, "UInt",&var2)
			DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8)), NumPut(0,var1,8)
			DllCall("oleaut32\SysFreeString", "UInt",NumGet(var2,8)), NumPut(0,var2,8)
		}
	Return 
	DDBAdd:
		Gosub,DDBAllocKey
		Gosub,DDBAllocItm
		DllCall(NumGet(NumGet(pDic%dic%+0)+40), "UInt",pDic%dic%, "UInt",&var1, "UInt",&var2) 
		DllCall("oleaut32\SysFreeString", "UInt",NumGet(var2,8)),DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
	Return
	DDBGet:
;~ 		OutputDebug % key
		Gosub,DDBAllocKey
		DllCall(NumGet(NumGet(pDic%dic%+0)+48), "UInt",pDic%dic%, "UInt",&var1, "IntP",bExist)
		If bExist {
			DllCall(NumGet(NumGet(pDic%dic%+0)+36), "UInt",pDic%dic%, "UInt",&var1, "UInt",&var2)
			nLen := DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var2,8), "Int",-1, "UInt",0, "Int",0, "UInt",0, "UInt",0)
			VarSetCapacity(Item, nLen,0)
			DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var2,8), "Int",-1, "Str",Item, "Int",nLen, "UInt",0, "UInt",0)
			DllCall("oleaut32\SysFreeString", "UInt",NumGet(var2,8))
		}
		DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
	Return Item
	DDBSet:
		Gosub,DDBAllocKey
		Gosub,DDBAllocItm
		DllCall(NumGet(NumGet(pDic%dic%+0)+32), "UInt",pDic%dic%, "UInt",&var1, "UInt",&var2)  ; 8 (Set0 -> 7)
		DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8)),DllCall("oleaut32\SysFreeString", "UInt",NumGet(var2,8))
	Return
	DDBExists:
		If !pdic%dic%
			Return 0
		Gosub,DDBAllocKey
		DllCall(NumGet(NumGet(pDic%dic%+0)+48), "UInt",pDic%dic%, "UInt",&var1, "IntP",bExist)
		DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
	Return bExist
	DDBAllocItm: ;Internal
		VarSetCapacity(wStr, StrLen(Item)*2+1,0)
		DllCall("MultiByteToWideChar", "UInt",0, "UInt",0, "UInt",&Item, "Int",-1, "UInt",&wStr, "Int",StrLen(Item)+1)
		NumPut(DllCall("oleaut32\SysAllocString","Str",wStr),var2,8)
	return
	DDBAllocKey: ;Internal
		VarSetCapacity(wStr, StrLen(Key)*2+1,0)
		DllCall("MultiByteToWideChar", "UInt",0, "UInt",0, "UInt",&Key, "Int",-1, "UInt",&wStr, "Int",StrLen(Key)+1)
		NumPut(DllCall("oleaut32\SysAllocString","Str",wStr),var1,8)
	Return
	DDBFindAll:
		If data
			wItem:=data
		else
			wItem:=Key
		DllCall(NumGet(NumGet(pDic%dic%+0)+80), "UInt",pDic%dic%, "UIntP",penum) ; create key-list in penum
		While % (penum 
				&& !(InStr("Clear0",pDic%dic%) 
				|| DllCall(NumGet(NumGet(penum+0)+12), "UInt",penum, "UInt",1, "UInt",&var1, "UInt",0))){
			nLen := DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "UInt",0, "Int",0, "UInt",0, "UInt",0)
			VarSetCapacity(wStr, nLen)
			DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "Str",wStr, "Int",nLen, "UInt",0, "UInt",0)
			key:=wStr
			NumPut(0,var1,8)
			Gosub, DDBGet
			result.=item . wItem
			DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
			NumPut(0,var1,8),NumPut(0,var2,8)
		}
		If (wItem!="")
			StringTrimRight,result,result,1
		DllCall(NumGet(NumGet(penum+0)+8), "UInt",penum)
	Return result
	DDBFind:
	DDBMatch:
	DDBRegEx:
	DDBGetAll:
	DDBCopy:
	DDBSort:
	DDBRegexReplace:
	DDBReplace:
		wKey:=key,wItem:=Item
		DllCall(NumGet(NumGet(pDic%dic%+0)+80), "UInt",pDic%dic%, "UIntP",penum) ; create key-list in penum
		While % (penum 
				&& !(InStr("Clear0",pDic%dic%) 
				|| DllCall(NumGet(NumGet(penum+0)+12), "UInt",penum, "UInt",1, "UInt",&var1, "UInt",0))){
			nLen := DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "UInt",0, "Int",0, "UInt",0, "UInt",0)
			VarSetCapacity(key, nLen,0)
			DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "Str",key, "Int",nLen, "UInt",0, "UInt",0)
			If (InStr("DDBFindDDBMatchDDBRegex",label) and (label="DDBFind" 
				? (InStr(Key,wKey)=1) : (label="DDBMatch" ? (InStr(key,wKey)) : (RegExMatch(key,wKey))))){
				bExist++
				If (bExist>skip){
					Gosub, DDBGet
					NumPut(0,var1,8),NumPut(0,var2,8)
					result.= Item . wItem
					nLenItm++
					If (limit<=nLenItm){
						DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
						Break
					}
				}
		} else if (label="DDBReplace"){
				Gosub, DDBGet
				NumPut(0,var1,8)
				result:=Item
				StringReplace,result,result,%wKey%,%wItem%,All
				Item:=result
				Gosub, DDBSet
				NumPut(0,var1,8),NumPut(0,var2,8)
			} else if (label="DDBRegexReplace"){
				Gosub, DDBGet
				NumPut(0,var1,8)
				Item:=RegExReplace(Item,wKey,wItem,"",-1)
				Gosub, DDBSet
				NumPut(0,var1,8),NumPut(0,var2,8)
			} else if !InStr("DDBFindDDBMatchDDBRegex",label){
				result.= Key . (label="DDBGetAll" ? wKey : (Chr(2) . DDB(dic,"get",Key) . Chr(3)))
			}
			DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
		}
		If (wItem!="" or (label="DDBGetAll" and wKey!=""))
			StringTrimRight,result,result,1
		DllCall(NumGet(NumGet(penum+0)+8), "UInt",penum)
		If InStr("DDBGetAllDDBFindDDBMatchDDBRegEx",label)
			Return result
		else if (label="DDBReplace"){
			Return
		} else if (label="DDBCopy"){
			Gosub, DDBGetCompMode
			dic:=wKey
	 } else {
		  Sort,result,% "D" . Chr(3) . wKey
		  Gosub, DDBDeleteAll
		}
		If (label="DDBCopy"){
			Key:= nCompMode ? nCompMode : wItem
			Gosub, DDBCreate
		}
		Loop,Parse,result,% Chr(3)
			If (A_LoopField!=""){
				key:=SubStr(A_LoopField,1,InStr(A_LoopField,Chr(2))-1)
				Item:=SubStr(A_LoopField,InStr(A_LoopField,Chr(2))+1)
				Gosub, DDBAdd
				NumPut(0,var1,8),NumPut(0,var2,8)
			}
	Return
	DDB&:
	DDBEnum:
		If Key>0
			penum%dic%:=Key
		else If (!penum%dic% or (Key="0" and !DllCall(NumGet(NumGet(penum%dic%+0)+8), "UInt",penum%dic%)))
			DllCall(NumGet(NumGet(pDic%dic%+0)+80), "UInt",pDic%dic%, "UIntP",penum%dic%) ; create key-list in penum
		If (InStr("Clear0",pDic%dic%) || DllCall(NumGet(NumGet(penum%dic%+0)+12), "UInt",penum%dic%, "UInt",1, "UInt",&var1, "UInt",0)) {
			DllCall(NumGet(NumGet(penum%dic%+0)+8), "UInt",penum%dic%)   ; END: destroy key-list
			penum%dic%=                                                  ; signal end of list
			ErrorLevel=                                                  ; empty
			Return      		                                           
		}
		nLen := DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "UInt",0, "Int",0, "UInt",0, "UInt",0)
		VarSetCapacity(wKey, nLen)
		DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "Str",wKey, "Int",nLen, "UInt",0, "UInt",0)
		DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
		ErrorLevel:=penum%dic%
	Return wKey
	DDBEnumerate:
		If Key>0
			penum%dic%:=Key
		else If (!penum%dic% or (Key="0" and !DllCall(NumGet(NumGet(penum%dic%+0)+8), "UInt",penum%dic%)))
			DllCall(NumGet(NumGet(pDic%dic%+0)+80), "UInt",pDic%dic%, "UIntP",penum%dic%) ; create key-list in penum
		If (InStr("Clear0",pDic%dic%) || DllCall(NumGet(NumGet(penum%dic%+0)+12), "UInt",penum%dic%, "UInt",1, "UInt",&var1, "UInt",0)) {
			DllCall(NumGet(NumGet(penum%dic%+0)+8), "UInt",penum%dic%)   ; END: destroy key-list
			penum%dic%=                                                  ; signal end of list
			ErrorLevel=                                                  ; empty
			Return      		                                           
		}
		nLen := DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "UInt",0, "Int",0, "UInt",0, "UInt",0)
		VarSetCapacity(wKey, nLen)
		DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "Str",wKey, "Int",nLen, "UInt",0, "UInt",0)
		DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
		ErrorLevel:=wKey
	Return penum%dic%
	DDBHash:
		Gosub, DDBAllocKey
		DllCall(NumGet(NumGet(pDic%dic%+0)+48), "UInt",pDic%dic%, "UInt",&var1, "IntP",bExist)
		DllCall(NumGet(NumGet(pDic%dic%+0)+84), "UInt",pDic%dic%, "UInt",&var1, "UInt",&var2)
		result := NumGet(var2,8)
		DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
		Return result
	Return
	DDB~:
	DDBRename:
		Gosub,DDBAllocKey
		Gosub,DDBAllocItm
		DllCall(NumGet(NumGet(pDic%dic%+0)+56), "UInt",pDic%dic%, "UInt",&var1, "UInt",&var2)
		DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
		DllCall("oleaut32\SysFreeString", "UInt",NumGet(var2,8))
	Return
	DDBDD:
	DDBD1:
	DDBD-:
	key:=data
	DDBDelete:
		Gosub,DDBAllocKey
		DllCall(NumGet(NumGet(pDic%dic%+0)+64), "UInt",pDic%dic%, "UInt",&var1)
		DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
	Return
	DDBDeleteAll:
	Return DllCall(NumGet(NumGet(pDic%dic%+0)+68), "UInt",pDic%dic%)
	DDBCount:
		DllCall(NumGet(NumGet(pDic%dic%+0)+44), "UInt",pDic%dic%, "IntP",result)
	Return result
	DDBDestroy:
		DllCall(NumGet(NumGet(pDic%dic%+0)+8), "UInt",pDic%dic%)
		pDic%dic%:="",key:=dic,dic:=""
		Goto,DDBDelete
	Return
	DDBCreate:
		If !Init
			DllCall("ole32\CoInitialize", "UInt",0),Init:=1
			,VarSetCapacity(var1, 16, 0),VarSetCapacity(var2, 16, 0),NumPut(8, var1),NumPut(8, var2)
		If (pdic%dic%=""){
			VarSetCapacity(CLSID, 16)
			VarSetCapacity(wKey, 79)
			DllCall("MultiByteToWideChar", "UInt",0, "UInt",0, "UInt",&CLSIDString, "Int",-1, "UInt",&wKey, "Int",39)
			DllCall("ole32\CLSIDFromString", "Str",wKey, "Str",CLSID)
			VarSetCapacity(IID, 16)
			DllCall("MultiByteToWideChar", "UInt",0, "UInt",0, "UInt",&IIDString, "Int",-1, "UInt",&wKey, "Int",39)
			DllCall("ole32\CLSIDFromString", "Str",wKey, "Str",IID)
			DllCall("ole32\CoCreateInstance", "Str",CLSID, "UInt",0, "UInt",5, "Str",IID, "UIntP",pDic%dic%) ; CLSCTX=5
			DllCall(NumGet(NumGet(pDic%dic%+0)+72), "UInt",pDic%dic%, "Int",Key) ; Set compare mode
		}
		key:=dic, Item:=pdic%dic%,dic:=""
		Gosub,DDBSet
		dic:=key
	Return pdic%dic%
	DDBGetCompMode:
		DllCall(NumGet(NumGet(pDic%dic%+0)+76), "UInt",pDic%dic%, "IntP",nCompMode)
	return nCompMode
	DDBDestroyAll:
		DllCall(NumGet(NumGet(pDic+0)+80), "UInt",pDic, "UIntP",penum) ; create key-list in penum
		While % (penum 
				&& !(InStr("Clear0",pDic) 
				|| DllCall(NumGet(NumGet(penum+0)+12), "UInt",penum, "UInt",1, "UInt",&var1, "UInt",0))){
			nLen := DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "UInt",0, "Int",0, "UInt",0, "UInt",0)
			VarSetCapacity(wStr, nLen)
			DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "Str",wStr, "Int",nLen, "UInt",0, "UInt",0)
			key:=wStr
			If key=+++
				continue
			Gosub, DDBGet
			IfInString,key,+
			{
				key:=SubStr(key,1,Instr(key,"+")-1)
				valued_%key%=
				delimiter_%key%=
				continue
			}
			pdic%key%=
			If Item
				DllCall(NumGet(NumGet(Item+0)+8), "UInt",Item)
			DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
			NumPut(0,var1,8)
		}
		DllCall(NumGet(NumGet(penum+0)+8), "UInt",penum)
	Return
	DDB:
	 If (!Init){
		DllCall("ole32\CoInitialize", "UInt",0),Init:=1
		,VarSetCapacity(var1, 16, 0),VarSetCapacity(var2, 16, 0),NumPut(8, var1),NumPut(8, var2)
		VarSetCapacity(wKey, 79)
		VarSetCapacity(CLSID, 16)
		DllCall("MultiByteToWideChar", "UInt",0, "UInt",0, "UInt",&CLSIDString, "Int",-1, "UInt",&wKey, "Int",39)
		DllCall("ole32\CLSIDFromString", "Str",wKey, "Str",CLSID)
		VarSetCapacity(IID, 16)
		DllCall("MultiByteToWideChar", "UInt",0, "UInt",0, "UInt",&IIDString, "Int",-1, "UInt",&wKey, "Int",39)
		DllCall("ole32\CLSIDFromString", "Str",wKey, "Str",IID)
		VarSetCapacity(CLSID_, 16)
		DllCall("MultiByteToWideChar", "UInt",0, "UInt",0, "UInt",&CLSID_StdGlobalInterfaceTable, "Int",-1, "UInt",&wKey, "Int",39)
		DllCall("ole32\CLSIDFromString", "Str",wKey, "Str",CLSID_)
		VarSetCapacity(IID_, 16)
		DllCall("MultiByteToWideChar", "UInt",0, "UInt",0, "UInt",&IID_IGlobalInterfaceTable, "Int",-1, "UInt",&wKey, "Int",39)
		DllCall("ole32\CLSIDFromString", "Str",wKey, "Str",IID_)
		DllCall("ole32\CoCreateInstance", "Str",CLSID_, "UInt",0, "UInt",5, "Str",IID_, "UIntP",pgit) ; CLSCTX=5
		DllCall(NumGet(NumGet(1*pgit)+20), "Uint", pgit, "Uint", 256, "Uint", &IID, "UintP", pDic)
		If (pDic=0){
			DllCall("ole32\CoCreateInstance", "Str",CLSID, "UInt",0, "UInt",5, "Str",IID, "UIntP",pDic) ; CLSCTX=5
			DllCall(NumGet(NumGet(pDic+0)+72), "UInt",pDic, "Int",1) ; Set compare mode
			key=+++
			Item:=pdic
			Gosub, DDBAdd
			key:="",Item:=""
			DllCall(NumGet(NumGet(1*pgit)+12), "Uint", pgit, "Uint", pDic, "Uint", &IID, "UintP", dwCookie)
		}
		Key=+++
		Gosub, DDBGet
		If (Item){
			pdic:=Item
			DllCall(NumGet(NumGet(pDic+0)+80), "UInt",pDic, "UIntP",penum) ; create key-list in penum
			While % (penum 
					&& !(InStr("Clear0",pDic) 
					|| DllCall(NumGet(NumGet(penum+0)+12), "UInt",penum, "UInt",1, "UInt",&var1, "UInt",0))){
				nLen := DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "UInt",0, "Int",0, "UInt",0, "UInt",0)
				VarSetCapacity(wStr, nLen)
				DllCall("WideCharToMultiByte", "UInt",0, "UInt",0, "UInt",NumGet(var1,8), "Int",-1, "Str",wStr, "Int",nLen, "UInt",0, "UInt",0)
				key:=wStr
				Gosub, DDBGet
				
				;Find function pointer
				label := DDB_()
				Loop {
					While % (n:=Chr(NumGet(NumGet(label+0), A_Index-1, "Char")))
						 Name .= n
					if (Name = key or Name = SubStr(key,1,InStr(key,"+")-1)) ; pFunc->mName
						break
					name=
					if !(label := NumGet(label+44)) ; pFunc->mNextFunc
						break
				}
				If InStr(key,"+valued")
				{
					StringTrimRight,key,key,7
					valued_%key%:=1
					If (name!=""){
						If NumGet(NumGet(NumGet(label+_Param)+16+8)+0,0,"Chr"){
							Loop 4
								NumPut(255,NumGet(NumGet(label+_Param)+16+8),A_Index-1,"Int")
						}
					}
					name=
					continue
				} else If InStr(key,"+delimiter"){
					StringTrimRight,key,key,10
					delimiter_%key%:=Item
					If (name!=""){
						If NumGet(NumGet(NumGet(label+_Param)+0+8)+0,0,"Chr"){
							NumPut(0,NumGet(NumGet(label+_Param)+0+8),A_Index-1,"Int")
						}
					}
					name=
					continue
				}
				
				If (!wStr or !IsFunc(wStr))
					Continue
				
				;check for parameters and empty those
				MaxParams:=NumGet(label+12,0)
				If (MaxParams=3){
					vName=
					pvName:=NumGet(NumGet(label+_Param,0)+16,8)
					vName := DllCall("MulDiv","uint",pvName,"int",1,"int",1,"str")
					If vName>0
						Loop 4
							NumPut(255,pvName,A_Index-1,"Int")

					vName=
					pvName:=NumGet(NumGet(label+_Param,0)+32,8)
					vName := DllCall("MulDiv","uint",pvName,"int",1,"int",1,"str")
					If vName>0
						NumPut(0,pvName+0,0)
				} else if (MaxParams=1){
					vName=
					pvName:=NumGet(NumGet(label+_Param,0)+0,8)
					vName := DllCall("MulDiv","uint",pvName,"int",1,"int",1,"str")
					If vName>0
						NumPut(0,pvName+0,0)
				}
				
				pdic%key%:=Item
				DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8))
				NumPut(0,var1,8)
			}
			DllCall(NumGet(NumGet(penum+0)+8), "UInt",penum)
		}
		pFunc := label := DDB_()
		Loop {
			While % (n:=Chr(NumGet(NumGet(label+0), A_Index-1, "Char")))
				 Name .= n
		  if (Name = A_ThisFunc) ; pFunc->mName
				break
			name=
		  if ! label := NumGet(label+44) ; pFunc->mNextFunc
				break
		}
		While % (A_Index < 2 or (pfunc:=NumGet(pFunc+44))){
			penum=
			pName:=NumGet(pFunc+0, 0, "UInt")
			Name := DllCall("MulDiv","uint",pName,"int",1,"int",1,"str")
			Key:=name
			Item=
			Gosub, DDBGet
			DllCall("oleaut32\SysFreeString", "UInt",NumGet(var1,8)),NumPut(0,var1,8)
			If (Item){
				pDic%name%:=Item
			}
			If (NumGet(NumGet(pFunc+4))=109){
				MaxParams:=NumGet(pFunc+12,0)
				If (MaxParams=3){
					pvName:=NumGet(NumGet(pFunc+_Param,0)+32,8)
					vName := DllCall("MulDiv","uint",pvName,"int",1,"int",1,"str")
					If vName>0
						Key:=vName, NumPut(0,pvName+0,0)
					else key:=0
				} else {
					key:=2
				}
				If (!item){
					dic:=name
					Gosub, DDBCreate
					Item:=pdic%name%,dic:=""
					Key:=name
					Gosub, DDBAdd
				}
				If !cg
					cg := DllCall("GlobalAlloc","uint",0x40,"uint",80)
				eline:=DDB_(cg)
				DDB_(cg,1)
				DDB_(cg,2)
				arg := NumGet(eline+0, 4)
				count += 1
				If (MaxParams=3){
					vName=
					pvName:=NumGet(NumGet(NumGet(pFunc+_Param,0)+32,0)+24,0)
					vName := DllCall("MulDiv","uint",pvName,"int",1,"int",1,"str")
					text%count% := "DDB(" name ",,,,,9223372036854775807," vName ","
					
					vName=
					pvName:=NumGet(NumGet(NumGet(pFunc+_Param,0)+0,0)+24,0)
					vName := DllCall("MulDiv","uint",pvName,"int",1,"int",1,"str")
					text%count% .= vName ","
					
					vName=
					pvName:=NumGet(NumGet(NumGet(pFunc+_Param,0)+16,0)+24,0)
					vName := DllCall("MulDiv","uint",pvName,"int",1,"int",1,"str")
					text%count% .= vName ")"
					
					VarSetCapacity(postfix%count%, 16*(9+2)+12, 0) ; 16*(MAX_PARAMS+2) + 12*1
				} else if (MaxParams=1) {
					vName=
					pvName:=NumGet(NumGet(NumGet(pFunc+_Param,0)+0,0)+24,0)
					vName := DllCall("MulDiv","uint",pvName,"int",1,"int",1,"str")
					text%count%:="DDB(" name ",,,,,9223372036854775807," vName ")"
					
					VarSetCapacity(postfix%count%, 16*(7+2)+12, 0) ; 16*(MAX_PARAMS+2) + 12*1
				}
				this_postfix := &postfix%count%
				number=9223372036854775807
				NumPut(DDB_("A_ThisFunc",3), this_postfix+0), NumPut(SYM_DYNAMIC, this_postfix+8)

				this_postfix += 80
				NumPut(&number, this_postfix+0), NumPut(0, this_postfix+8)
			  ; Move to next token in postfix%count% array
				this_postfix += 16
				If (MaxParams=3){
					
					Array_var := NumGet(NumGet(pFunc+_Param)+32)
					; Put param var into postfix%count% array.
					NumPut(Array_var, this_postfix+0), NumPut(SYM_VAR, this_postfix+8)
					this_postfix += 16
					
					Array_var := NumGet(NumGet(pFunc+_Param)+0)
					; Put param var into postfix%count% array.
					NumPut(Array_var, this_postfix+0), NumPut(SYM_VAR, this_postfix+8)
					this_postfix += 16
					
					; Put param var into postfix%count% array.
					Array_var := NumGet(NumGet(pFunc+_Param)+16)
					NumPut(Array_var, this_postfix+0), NumPut(SYM_VAR, this_postfix+8)
					
					If NumGet(NumGet(NumGet(pFunc+_Param)+16+8)+0,0,"Chr"){
						valued_%name%:=1
						Item:=1,dic:=""
						Key:=name . "+valued"
						Gosub, DDBSet
						Loop 4
							NumPut(255,NumGet(NumGet(pFunc+_Param)+16+8),A_Index-1,"Int")
					}
					
				} else if (MaxParams=1) {
					Array_var := NumGet(NumGet(pFunc+_Param)+0)
					; Put param var into postfix%count% array.
					NumPut(Array_var, this_postfix+0), NumPut(SYM_VAR, this_postfix+8)
					
					If (delimiter_%name%:=Chr(NumGet(NumGet(NumGet(pFunc+_Param)+0+8)+0,0,"Chr"))){
						Item:=delimiter_%name%,dic:=""
						Key:=name . "+delimiter"
						Gosub, DDBSet
						NumPut(0,NumGet(NumGet(pFunc+_Param)+0+8),0,"Int")
					}
				}
				; Move to next token in postfix%count% array, increment param count.
				this_postfix += 16

				; Set up the DerefType at the end of the postfix%count% array.
				func_deref := this_postfix+32
				NumPut(label, func_deref+4)
				If (MaxParams=3)
					NumPut(9, func_deref+9, 0, "char")
				else if (MaxParams=1)
					NumPut(7, func_deref+9, 0, "char")

				; Set up SYM_FUNC token.
				NumPut(func_deref, this_postfix+0)
				NumPut(SYM_FUNC, this_postfix+8)
				this_postfix += 16
				; Terminate postfix%count% array.
				NumPut(SYM_INVALID, this_postfix+8)

				; Set postfix%count% array of the first arg of 'return' below.
				NumPut(&postfix%count%, arg+12)
				NumPut(&text%count%, arg+4)
				If (ch := DDB_(cg,4))
				{
					; Re-target the function at the new line.
					NumPut(eline, pFunc+_JumpToLine)
					DllCall("GlobalFree","uint",ch)
				}
			}
			name=
		}
 } else {
		Gosub, DDBDestroyAll
		Init=
		DllCall(NumGet(NumGet(pDic+0)+8), "UInt",pDic)
		pdic=
		DllCall("ole32\CoUnInitialize")
	 }
	Return
}



DDB_(cg="",info=""){
    static pFirstFunc, action_type:=102, pool, line_struct_size, type:=0,param:=1,empty,pDerefs, DerefCount
	 If (cg="" and Info=""){
		 if !pFirstFunc {
			if pCb := RegisterCallback(A_ThisFunc){
				func := NumGet(pCb+28)
				DllCall("GlobalFree","uint",pCb)
				if pFirstLine := NumGet(func+4) ; mJumpToLine
				Loop {
					 if !(pLine:=NumGet(pFirstLine+16)) ; mPrevLine
						  break
					 pFirstLine := pLine
				}
				pLine :=pFirstLine
				Loop {
					Loop % NumGet(pLine+1,0,"uchar") { ; pLine->mArgc
						 pArg := NumGet(pLine+4) + (A_Index-1)*16  ; pLine->mArg[A_Index-1]
						 if (NumGet(pArg+0,0,"uchar") != 0) ; pArg->type != ARG_TYPE_NORMAL
							  continue ; arg has no derefs (only a Var*)
						 Loop {
							  pDeref := NumGet(pArg+8) + (A_Index-1)*12  ; pArg->deref[A_Index-1]
							  if (!NumGet(pDeref+0)) ; pDeref->marker (NULL terminates list)
									break
							  if (NumGet(pDeref+8,0,"uchar")) ; pDeref->is_function
							  {
									; The first function is either the first defined function,
									; or if no explicitly #included UDFs exist, the first
									; built-in function referenced in code.
									pFunc := NumGet(pDeref+4)
									if (NumGet(pFunc+49,0,"uchar")) { ; pFunc->mIsBuiltIn
										 if !pFirstBIF
											  pFirstBIF := pFunc
									} else { ; UDF
										 pFuncLine := NumGet(pFunc+4)
										 FuncLine := NumGet(pFuncLine+8)
										 FuncFile := NumGet(pFuncLine+2,0,"ushort")
										 if !pFirstFunc or (FuncFile < FirstFuncFile || (FuncFile = FirstFuncFile && FuncLine < FirstFuncLine))
											  pFirstFunc:=pFunc, FirstFuncLine:=FuncLine, FirstFuncFile:=FuncFile
									}
							  }
						 }
					}
					if !(pLine:=NumGet(pLine+20)) ; pLine->mNextLine
						 break
				}
				if pFirstBIF
				{   ; Usually the first UDF will be before the first BIF, but not if
					; only auto-included/stdlib UDFs exist *AND* the first BIF is
					; referenced in code before the first UDF.
					if pFirstFunc
					{   ; Look for the BIF using the UDF as a starting point.
						 pFunc := pFirstFunc
						 Loop {
							  if !(pFunc := NumGet(pFunc+44)) ; pFunc->mNextFunc
									break
							  ; If the BIF is found, the UDF must precede it in the list.
							  if (pFunc = pFirstBIF)
									return pFirstFunc
						 }
					; If we got here, the BIF was not found, so is probably the first Func.
					}
					pFirstFunc := pFirstBIF
					return pFirstFunc
				}
			}
		}
		return pFirstFunc
	} else if (cg="" and info=1){ ;getlabel buildinvars
		if !pFirstLabel {
			  if !(pLine := NumGet(DDB_()+4))
					return 0
			  Loop {
					act := NumGet(pLine+0,0,"char")
					if (act = 96 || act = 95) ; ACT_GOSUB || ACT_GOTO
						 break
					if !(pLine:=NumGet(pLine+20)) ; pLine->mNextLine
						 return 0
			  }
			  pFirstLabel := NumGet(pLine+24)
			  Loop {
					if ! pPrevLabel:=NumGet(pFirstLabel+8)
						 break
					pFirstLabel := pPrevLabel
			  }
		 }
		 pLabel := pFirstLabel
		 Loop {
			  if DllCall("MulDiv","uint",NumGet(pLabel+0),"int",1,"int",1,"str") = "__getBuiltInVar_A_ThisFunc"
					return pLabel
			  if ! pLabel := NumGet(pLabel+12)
					return 0
		 }
	} else if (cg and info=4){ ;finalize
		; Finalize last_line, if applicable.
		 
		 ; Ensure each func/label has a target line. Might help catch errors.
		 ; These loops are also required to determine the last func/label if (flags & 1).
		 label := NumGet(cg+20)
		 label_count := 0
		 Loop {
			  if !label
					break
			  label_count += 1
			  label := NumGet(label+12)
		 }
		 func := NumGet(cg+24)
		 func_count := 0
		 Loop {
			  if !func
					break
			  func_count += 1
			  func := NumGet(func+44)
		 }
		 
		 if NumGet(cg+76) & 2 ; needs_resolve: label or function references need to be resolved.
		 {
				line := NumGet(cg+16)
				 Loop
				 {
					  arg_count := NumGet(line+1,0,"uchar")
					  
					  ; Resolve as-yet-unresolved function calls.
					  arg := NumGet(line+4)
					  Loop %arg_count%
					  {
							if NumGet(arg+0,0,"uchar") = 0 && NumGet(arg+1,0,"uchar")
							{   ; This arg is an expression.
								 ; AutoHotkey v1.0.48: Expressions are now tokenized at load-time.
								 if !NumGet(arg+12)
									  return false, ErrorLevel:="Expression has not been pre-tokenized."
							}
							arg += 16
					  }
					  ; Resolve static labels for Goto, Gosub, OnExit, Hotkey, SetTimer and GroupAdd.
					  line_action := NumGet(line+0,0,"uchar")
					  if (line_action >= 95 && line_action <= 99 || line_action = 140)
							&& arg_count >= (line_action=98 ? 2 : line_action=140 ? 4 : 1)
					  {
							label_name := ""
							arg := NumGet(line+4)
							deref := NumGet(arg+8)
							
							if line_action = 140 ; GroupAdd
							{
								 ; GroupAdd's fourth parameter may be a label.
								 deref := NumGet(arg+56)
								 if NumGet(arg+48,0,"uchar") = 0 && !(deref && NumGet(deref+0))
									  label_name := DllCall("MulDiv","uint",NumGet(arg+52),"int",1,"int",1,"str")
							}
							else if NumGet(arg+0,0,"uchar") = 0 && !(deref && NumGet(deref+0))
							{   ; arg 1 is ARG_TYPE_NORMAL and has no derefs
							
								 if line_action = 98 ; Hotkey
								 {
									  deref := NumGet(arg+24)
									  if NumGet(arg+12,0,"uchar") = 0 && !(deref && NumGet(deref+0)) ; second arg has no derefs
											&& SubStr(DllCall("MulDiv","uint",NumGet(arg+4),"int",1,"int",1,"str"),1,5) != "IfWin"
											; and first arg is not IfWin-something
									  {
											label_name := DllCall("MulDiv","uint",NumGet(arg+20),"int",1,"int",1,"str")
									  }
								 }
								 else
									  label_name := DllCall("MulDiv","uint",NumGet(arg+4),"int",1,"int",1,"str")
							}
							
							if label_name
							{
								 ; Call %OnResolveLabel%().
								 label := %OnResolveLabel%(cg, label_name)
								 
								 ; Treat this as non-critical, since
								 ;   a) the parameter might be a non-label (On, Off, etc.) and
								 ;   b) AutoHotkey will attempt to resolve the label at run-time.
								 if label is not integer
									  label := 0

								 NumPut(label, line + (line_action=95||line_action=96||line_action=140 ? 24 : 12))
							}
					  }
					  
					  if !(line := NumGet(line+20))
							break
				 }
;~ 				 return true

			  NumPut(NumGet(cg+76) & ~2, cg+76) ; remove flag: needs_resolve.
		 }
;~ 		 code_process_blocks(NumGet(cg+16))
;~ 		 if ErrorLevel
;~ 			  return 0
		 
		 hcode := DllCall("GlobalAlloc","uint",0,"uint", 28 + func_count*4 + label_count*4)
		 if !hcode
			  return 0, ErrorLevel:="MEM"
		 
		 ; For maintainability, don't assign something meaningless if count=0.
		 func_list := func_count ? hcode + 28 : 0
		 label_list := label_count ? hcode + 28 + func_count*4 : 0
		 
		 ; Copy the labels and functions into an array, in case
		 ; the script later alters or breaks the linked list(s).
		 label := NumGet(cg+20)
		 Loop {
			  if !label
					break
			  NumPut(label, label_list + A_Index*4-4)
			  label := NumGet(label+12)
		 }
		 func := NumGet(cg+24)
		 Loop {
			  if !func
					break
			  NumPut(func, func_list + A_Index*4-4)
			  func := NumGet(func+44)
		 }
		 
		  NumPut(NumGet(cg+16), hcode+0) ; first_line
		 ,NumPut(NumGet(cg+4), hcode+4) ; last_line
		 ,NumPut(flags, hcode+8)
		 ,NumPut(label_list, NumPut(label_count, hcode+12))
		 ,NumPut(func_list, NumPut(func_count, hcode+20))
		 
		 ; Reset last_line/label/func and first_line/label/func.
		 NumPut(0,NumPut(0,NumPut(0,NumPut(0,NumPut(0,NumPut(0,cg+4))))))
		 ; Reset flags.
		 , NumPut(0,cg+76)
		 
		 
		 
		 if flags & 1 ; Add labels and functions to the main script.
		 {
			  if func_count
			  {
					func_1 := DDB_() ;__getFirstFunc()
					func_2 := NumGet(func_1+44)
					NumPut(func_2, NumGet(func_list+func_count*4-4)+44)
					NumPut(NumGet(func_list+0), func_1+44)
			  }
		 }
		 
		 return hcode
	} else if (cg and info=3){
		 if !pDerefs ;= label->mJumpToLine->mArg[0]->deref
		 {
			  pDerefs := NumGet(NumGet(NumGet(DDB_("",1),4),4),8)
			  Loop
					if ! NumGet(pDerefs+(A_Index-1)*12) {
						 DerefCount := A_Index-1
						 break
					}
		 }
		 low := 0
		 high := DerefCount - 1
		 Loop {
			  if (low > high)
					break
			  mid := (low+high)//2
			  i := DllCall("shlwapi\StrCmpNIA","uint",NumGet(pDerefs+mid*12),"str",cg,"int",NumGet(pDerefs+mid*12,10,"ushort"))
			  if i > 0
					high := mid - 1
			  else if i < 0
					low := mid + 1
			  else
					return NumGet(pDerefs+mid*12,4)
		 }
		 return 0
__getBuiltInVar_A_ThisFunc:
	 return % 0,
	 ( Join`s
A_ThisFunc
	 )
	} else if (cg and info=2){
		if ! this_line := NumGet(cg+4)
			  return 0, ErrorLevel:="Invalid usage (no line)."
		 if NumGet(this_line+1,0,"uchar")
			  return 0, ErrorLevel:="Invalid usage (line already has args)."
		 ; Finalize current_arg, if applicable.
		 if arg_count := NumGet(cg+52)
		 {   ; Move args from the buffer to this_line.
			  arg_ptr := DllCall("GlobalAlloc","uint",0,"uint",arg_count*16)
			  if !arg_ptr
					return 0, ErrorLevel:="MEM"
			  DllCall("RtlMoveMemory","uint",arg_ptr,"uint",NumGet(cg+60),"uint",arg_count*16)
			  , NumPut(0, cg+52) ; arg_count := 0
		 } else
			  arg_ptr := 0
		 NumPut(arg_count, this_line+1, "uchar"), NumPut(arg_ptr, this_line+4)
		 ; Determine line Attribute.
		 attr := 0
		 if NumGet(this_line+0,0,"UChar") = 104 ; ACT_LOOP
		 {   ; Parsing loops must be marked as such at load-time.
			  ; Other types of loops are handled by AutoHotkey at run-time.
			  ; Future optimization: detect other types of loops at load-time where possible.
			  if arg_count = 0
					attr := 2 ; ATTR_LOOP_NORMAL
			  else if arg_count >= 2
			  {   arg_str := DllCall("MulDiv","uint",NumGet(arg_ptr+4),"int",1,"int",1,"str") ;(NumGet(arg_ptr+4))
					if arg_str = Read
						 attr := 5 ; ATTR_LOOP_READ_FILE
					else if arg_str = Parse
						 attr := 6 ; ATTR_LOOP_PARSE
			  }
		 }
		 NumPut(attr, this_line+12)
		 return this_line
	} else if (cg and info=1){
;~ 			if ! code_expect_args(cg, 1 + arg_count:=NumGet(cg+52), 2)
;~ 			  return false, ErrorLevel:="MEM"
		buf_info_ptr:=cg+52
		min_buf_size:=1 + arg_count:=NumGet(cg+52)
		init_buf_size:=20
		max_buf_size:=20
		item_size:=16
		growth_factor:=2
		old_buf_size := NumGet(buf_info_ptr+4)
		old_buf      := NumGet(buf_info_ptr+8)
		; If the buffer is already large enough, no need to do anything.
		if !(old_buf && old_buf_size >= min_buf_size){
			; Allow caller to specify "exponential" growth to reduce the required
			; number of reallocations (e.g. as items are added one by one.)
			if growth_factor
			{
			  grown_buf_size := old_buf_size ? old_buf_size * growth_factor : init_buf_size
			  ; max_buf_size applies only to grown buffer size.
			  if (grown_buf_size > max_buf_size)
					grown_buf_size := max_buf_size
			  if (grown_buf_size > min_buf_size)
					min_buf_size := grown_buf_size
			}
			if old_buf
			  new_buf := DllCall("GlobalReAlloc","uint",old_buf,"uint",min_buf_size*item_size,"uint",0x2)
			else
			  new_buf := DllCall("GlobalAlloc","uint",0x0,"uint",min_buf_size*item_size)
			if !new_buf
			  return false
			NumPut(new_buf, NumPut(min_buf_size, buf_info_ptr+4))
		}
		; Update arg_ptr to point to new arg.
		NumPut(arg_ptr := NumGet(cg+60) + arg_count*16, cg+0)
		; ++arg_count
		NumPut(arg_count + 1, cg+52)
		; ZeroMemory(arg_ptr, 16)
		NumPut(0, NumPut(0, NumPut(0, NumPut(0, arg_ptr+0))))
		; Set type and is_expression.
		NumPut(type=0 && param, NumPut(type+0, arg_ptr+0,0,"char") ,0,"char")
		; Set Var* if this arg is a non-dynamic input/output var.
		; Set text="" as an indicator that the arg has a non-dynamic var.
		if (type = 1 || type = 2) && param
		  NumPut(param, NumPut(&empty, arg_ptr+4))	  
	} else {
		if !line_struct_size
			  line_struct_size := (InStr(A_AhkVersion,"l")) ? 36 : 32  ; Support for AutoHotkey_L.
		 if this_line
			  ; Clear the Line for consistency (returned lines will always be empty)
			  ; and to be sure ListLines doesn't attempt to access invalid args, etc.
			  DllCall("RtlZeroMemory","uint",this_line,"uint",32), pool .= this_line . ","
		 else if i:=InStr(pool,",")
			  ; Remove and return a Line from the pool.
			  this_line:=SubStr(pool,1,i-1), pool:=SubStr(pool,i+1)
		 else
		 {   ; No Lines currently in the pool, so allocate a bunch.
			  this_line := DllCall("GlobalAlloc","uint",0x40,"uint",32*line_struct_size)
			  pool =
			  Loop, 31    ; exclude the first Line.
					pool .= this_line+A_Index*line_struct_size . ","
		 }
		NumPut(102, this_line+0,0,"uchar")
		
		; Add line to linked list.
		if last_line := NumGet(cg+4)
		  NumPut(this_line, last_line+20), NumPut(last_line, this_line+16)
		else
		  NumPut(this_line, cg+16)    ; first_line := this_line

		NumPut(this_line, cg+4)
		; NOTE: All lines must be processed later to set mParentLine & mRelatedLine.
		;       See code_process_blocks.

		; Set mJumpToLine of last_label/last_func if not already set.
		; It is caller's responsibility to ensure the line is a valid target.
		if (last_func := NumGet(cg+12)) && ! NumGet(last_func+4)
		{
		  if NumGet(cg+76) & 1 ; next_line_is_function_body
				NumPut(this_line, last_func+4), NumPut(NumGet(cg+76) & ~1, cg+76)
		  else if action_type = 107 ; ACT_BLOCK_BEGIN
				NumPut(1, this_line+12), NumPut(NumGet(cg+76) | 1, cg+76)
		}
		if (last_label := NumGet(cg+8)) && ! NumGet(last_label+4)
		{
		  Loop {
				NumPut(this_line, last_label+4)
				if !(last_label := NumGet(last_label+8)) || NumGet(last_label+4)
					 break
		  }
		}
		return this_line
	}
}














